
#include "TArtAnaLoop.hh"

#include <iostream>
#include <limits>

#include "TThread.h"
#include "TObject.h"
#include "TDirectory.h"
#include "TSystem.h"
#include "TH1.h"

#include "TArtEventStore.h"
#include "TArtStoreManager.h"

#include "TArtAnaLoopManager.hh"
#include "TArtAnaFile.hh"

TArtAnaLoop::TArtAnaLoop()
  : fAnaFile(0), fEventStore(0), fNumEvents(0), fNumTotalEvents(0)
{;}

TArtAnaLoop::~TArtAnaLoop()
{
  delete fAnaFile;
  fAnaFile = 0;
  delete fEventStore;
  fEventStore = 0;
}

bool TArtAnaLoop::OpenAnaFile(const char* anafilename)
{
  if(fAnaFile){
    std::cout << "destruct current anafile" << std::endl;
    delete fAnaFile;
  }
  fAnaFile = new TArtAnaFile;
  if(!fAnaFile->Construct() ||
     !fAnaFile->Open(anafilename) ||
     !fAnaFile->Check()){
    std::cout << "anafile error" << std::endl;
    delete fAnaFile;
    fAnaFile = 0;
    return false;
  }else{
    return true;
  }
}

bool TArtAnaLoop::ChangeAnaFile(const char* anafilename)
{
  if(!fAnaFile){
    std::cout << "no anafile is opened. construct new anafile." << std::endl;
    return OpenAnaFile(anafilename);
  }

  fAnaFile->ClearVal();
  fAnaFile->ClearMap();
  if(!fAnaFile->Construct() ||
     !fAnaFile->Open(anafilename) ||
     !fAnaFile->Check()){
    std::cout << "anafile error" << std::endl;
    return false;
  }else{
    return true;
  }
}

void TArtAnaLoop::AnaLoop(const char* filename, int eventnumber)
{
  if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::INIT){
    fNumEvents = 0;

    fEventStore = new TArtEventStore;
    bool isopen;
    if(filename == 0){
      isopen = fEventStore->Open();
    }else{
      isopen = fEventStore->Open(filename);
    }
    if(!isopen){
      delete fEventStore;
      fEventStore = 0;
      TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::ERROR;
      return;
    }

    Construct();

    TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::RUNNING;
  }

  if(fEventStore == 0) return;

  if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::RUNNING){
    while(1){
      ++fNumEvents;
      ++fNumTotalEvents;
      bool nextevent = false;
      if(filename == 0){
	while(!(nextevent = fEventStore->GetNextEvent())){
	  if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::END){
	    break;
	  }else if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::SUSPENDED){
	    break;
	  }else{
	    Wait();
	  }
	}
      }else{
	nextevent = fEventStore->GetNextEvent();
      }
      if(!nextevent || (!(eventnumber<0) && fNumEvents>eventnumber)){
	std::cout << "\n\n analysis completed!! \n" << std::endl;
	TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::END;
	break;
      }else if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::END){
	break;
      }

      Calculate();
      fEventStore->ClearData();

      if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::SUSPENDED){
	break;
      }
    }
  }

  if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::END){
    Destruct();
    delete fEventStore;
    fEventStore = 0;
    delete TArtStoreManager::Instance();
  }
}

void TArtAnaLoop::Wait()
{
  gSystem->Sleep(100);
}
