
#include "TArtAnaLoopManager.hh"

#include <iostream>
#include <fstream>

#include "TThread.h"
#include "TDirectory.h"
#include "TList.h"
#include "TH1.h"
#include "TSystem.h"

#include "TArtAnaLoop.hh"
#include "TArtAnaFile.hh"

#include "TArtAnaClock.hh"

//_______________________________________________________________________________________


void book(TArtAnaLoop* analoop, const char* anafilename, const char* filename,
	  int eventnumber, TDirectory* directory)
{TArtAnaLoopManager::Book(analoop, anafilename, filename, eventnumber, directory);}
void push(const char* filename){TArtAnaLoopManager::PushRidf(filename);}
void pop(int i){TArtAnaLoopManager::PopRidf(i);}
void start(){TArtAnaLoopManager::Start();}
void stop(){TArtAnaLoopManager::Stop();}
void end(){TArtAnaLoopManager::End();}
void clear(){TArtAnaLoopManager::Clear();}
void status(){TArtAnaLoopManager::Status();}

TCutG* hcut(int id, const char* filename, const char* cutname){return TArtAnaLoopManager::HCut(id, filename, cutname);}

//_______________________________________________________________________________________

TArtAnaLoopManager::EAnalysisState TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::INIT;
TThread* TArtAnaLoopManager::fThread = 0;
TThread* TArtAnaLoopManager::fThreadNextRidf = 0;

std::deque<const char*> TArtAnaLoopManager::fNextRidfFileName;
std::deque<int> TArtAnaLoopManager::fNextRidfEventNumber;
std::deque<TDirectory*> TArtAnaLoopManager::fNextRidfDirectory;

TThread* TArtAnaLoopManager::Book(TArtAnaLoop* analoop,
				  const char* anafilename, const char* filename,
				  int eventnumber, TDirectory* directory)
{
  if(kAnalysisState == ERROR){
    std::cout << "\n recover from error\n" << std::endl;
    //    Clear();
    kAnalysisState = INIT;
  }

  if(kAnalysisState == END){
    kAnalysisState = INIT;
  }

  if(fThread != 0 ){
    if(kAnalysisState != INIT && kAnalysisState != END) End();
    delete fThread;
    fThread = 0;
  }

  //  TArtAnaClock::Start();

  TArtAnaLoop* analoop_last = TArtAnaLoopStatic::GetAnaLoop();

  if(!analoop){
    if(analoop_last){
      if(anafilename != "" && anafilename != 0){      
	std::cout << "\n restart analoop by new anafile\n" << std::endl;
	if(!analoop_last->ChangeAnaFile(anafilename)) return 0;
      }else{
	/* nothing */;
      }
    }else{
      std::cout << "\n no analoop is set\n" << std::endl;
      return 0;
    }
    analoop = analoop_last;
  }else{
    if(analoop_last){
      if(anafilename != "" && anafilename != 0){
	std::cout << "\n reset analoop and anafile\n" << std::endl;
	ClearHist();
	delete analoop_last;
	analoop_last = 0;
	if(!analoop->OpenAnaFile(anafilename)) return 0;
      }else{
	std::cout << "\n reset analoop\n" << std::endl;
	ClearHist();
	delete analoop_last;
	analoop_last = 0;
      }
    }else{
      if(anafilename != "" && anafilename != 0){
	if(!analoop->OpenAnaFile(anafilename)) return 0;
	std::cout << "\n set analoop and anafile\n" << std::endl;
      }else{
	/* nothing */
      }
    }
  }

//  TArtAnaClock::Stop();
//  std::cout << "\n time:" << std::endl;
//  TArtAnaClock::Print();

  TArtAnaLoopStatic::SetAnaLoop(analoop);
  TArtAnaLoopStatic::SetFileName(filename);
  TArtAnaLoopStatic::SetEventNumber(eventnumber);
  TArtAnaLoopStatic::SetDirectory(directory);

  void *(*fpt)(void*) = &TArtAnaLoopStatic::AnaLoopStatic;

  return fThread = new TThread((void *(*)(void*))fpt, (void*)0);
}

TThread* TArtAnaLoopManager::PushRidf(const char* filename, int eventnumber, TDirectory* directory)
{
  if(directory == 0) directory = gDirectory;

  if(fNextRidfFileName.empty()){
    delete fThreadNextRidf;
    fThreadNextRidf = 0;    
  }

  fNextRidfFileName.push_back(filename);
  fNextRidfEventNumber.push_back(eventnumber);
  fNextRidfDirectory.push_back(directory);

  if(fThreadNextRidf == 0){
    void *(*fpt)(void*) = &TArtAnaLoopManager::RunNextRidf;
    fThreadNextRidf = new TThread((void*(*)(void*))fpt,
				  (void*)0);
    fThreadNextRidf->Run();
  }

  return fThreadNextRidf;
}

void TArtAnaLoopManager::PopRidf(int i)
{
  if(i>(int)fNextRidfFileName.size() || i<1){
    std::cout << "\n cannot pop " << i << "\n" << std::endl;
    return;
  }
    
  fNextRidfFileName.erase(fNextRidfFileName.begin()+i-1);
  fNextRidfEventNumber.erase(fNextRidfEventNumber.begin()+i-1);
  fNextRidfDirectory.erase(fNextRidfDirectory.begin()+i-1);
}

void* TArtAnaLoopManager::RunNextRidf(void* /*ptr*/)
{
  while(1){
    if(kAnalysisState != END){
      while(kAnalysisState != END){
	if(gSystem) gSystem->Sleep(500);
      }
    }

    if(fNextRidfFileName.empty()) break;

    const char* filename = fNextRidfFileName[0];
    int eventnumber = fNextRidfEventNumber[0];
    TDirectory* directory = fNextRidfDirectory[0];
    fNextRidfFileName.pop_front();
    fNextRidfEventNumber.pop_front();
    fNextRidfDirectory.pop_front();

    if(!fThread){
      std::cout << "cannot push ridf. need to book." << std::endl;
      return 0;
    }    

    if(!Book(0, 0, filename, eventnumber, directory)){
      return 0;
    }else{
      std::cout << "start" << std::endl;
      Start();
    }
  }

  std::cout << "end" << std::endl;
  return 0;
}

//  TArtAnaClock::Stop();
//  std::cout << "\n time:" << std::endl;
//  TArtAnaClock::Print();
//  TArtAnaClock::Start();

void TArtAnaLoopManager::Start()
{
  if(!fThread){
    std::cout << "\n no anafile is set\n" << std::endl;
    return;
  }

  if(kAnalysisState == INIT){
    std::cout << "\n start analoop\n" << std::endl;    
    fThread->Run();
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n ...running\n" << std::endl;
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n resume analoop\n" << std::endl;    
    kAnalysisState = RUNNING;
    fThread->Run();
  }else if(kAnalysisState == END){
    std::cout << "\n start analoop\n" << std::endl;
    kAnalysisState = INIT;
    fThread->Run();
  }else if(kAnalysisState == ERROR){
    std::cout << "\n start analoop\n" << std::endl;    
    fThread->Run();
  }
}

void TArtAnaLoopManager::Stop()
{
  if(!fThread){
    std::cout << "\n no anafile is set\n" << std::endl;
    return;
  }

  if(kAnalysisState == INIT){
    std::cout << "\n ...init\n" << std::endl;
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n stop analoop\n" << std::endl;
    kAnalysisState = SUSPENDED;
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n ...suspended\n" << std::endl;    
  }else if(kAnalysisState == END){
    std::cout << "\n ...end\n" << std::endl;
  }else if(kAnalysisState == ERROR){
    std::cout << "\n ...error\n" << std::endl;    
    fThread->Run();
  }
}

void TArtAnaLoopManager::End()
{
  if(!fThread){
    std::cout << "\n no anafile is set\n" << std::endl;
    return;
  }

  if(kAnalysisState == INIT){
    std::cout << "\n ...init\n" << std::endl;
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n end analoop\n" << std::endl;
    kAnalysisState = END;
    fThread->Join();
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n end analoop\n" << std::endl;
    kAnalysisState = END;
    fThread->Run();
    fThread->Join();
  }else if(kAnalysisState == END){
    std::cout << "\n ...end\n" << std::endl;
    fThread->Join();
  }else if(kAnalysisState == ERROR){
    std::cout << "\n ...error\n" << std::endl;    
    fThread->Run();
    fThread->Join();
  }
}

void TArtAnaLoopManager::Clear()
{
  if(!kAnalysisState != END) End();

  TArtAnaLoop* analoop = TArtAnaLoopStatic::GetAnaLoop();
  if(analoop == 0) return;
  delete analoop;
  //  if(analoop->fAnaFile == 0) return;
  //  delete analoop->fAnaFile;

  TArtAnaLoopStatic::Clear();

  delete fThread;
  fThread = 0;

  ClearHist();

  kAnalysisState = INIT;
}

void TArtAnaLoopManager::ClearHist()
{
  if(!gDirectory) return;
  TList* list = gDirectory->GetList();
  if(!list) return;
  TObjLink* objlink = list->FirstLink();	 
  TObject* obj;
  while(objlink){
    obj = objlink->GetObject();
    objlink = objlink->Next();
    if(obj && obj->InheritsFrom(TH1::Class())){
      delete obj;
    }
  }
}

void TArtAnaLoopManager::Status()
{
  std::cout << std::endl;

  TString state;
  switch (kAnalysisState){
  case INIT:
    state = "INIT"; break;
  case RUNNING:
    state = "RUNNING"; break;
  case SUSPENDED:
    state = "SUSPENDED"; break;
  case END:
    state = "END"; break;
  case ERROR:
    state = "ERROR"; break;
  }
  std::cout << " Analysis State: " << state << std::endl;
  std::cout << " File Name: " << TArtAnaLoopStatic::GetFileName() << std::endl;
  TArtAnaLoop* analoop = TArtAnaLoopStatic::GetAnaLoop();
  TString classname("");
  int eventnumber(0);
  int totaleventnumber(0);
  if(analoop != 0){
    classname = analoop->ClassName();
    eventnumber = analoop->GetEventNumber();
    totaleventnumber = analoop->GetTotalEventNumber();
  }
  std::cout << " Class Name: " << classname << std::endl;
  std::cout << " Event Number: " << eventnumber << std::endl;
  std::cout << " Total Event Number: " << totaleventnumber << std::endl;
  std::cout << " Stack of RIDF: " << std::endl;
  for(int i=0; i<(int)fNextRidfFileName.size(); ++i){
    std::cout << "  " << i+1 << ": " << fNextRidfFileName[i] << std::endl;
  }

  std::cout << std::endl;
}

TCutG* TArtAnaLoopManager::HCut(int id, const char* filename, const char* cutname)
{
  TArtAnaLoop* analoop = TArtAnaLoopStatic::GetAnaLoop();
  if(analoop == 0) return 0;
  if(analoop->fAnaFile == 0) return 0;

  return analoop->fAnaFile->CreateCutG(id, filename, cutname);
}

TArtAnaLoopManager::~TArtAnaLoopManager()
{
  delete fThread;
  fThread = 0;
}

//_______________________________________________________________________________________

TArtAnaLoop* TArtAnaLoopStatic::fAnaLoop = 0;
TString TArtAnaLoopStatic::fFileName = "";
int TArtAnaLoopStatic::fEventNumber = -1;
TDirectory* TArtAnaLoopStatic::fDirectory = 0;

void* TArtAnaLoopStatic::AnaLoopStatic(void *)
{
  fDirectory->cd();
  if(fFileName.Length() == 0){
    fAnaLoop->AnaLoop((char*)0, fEventNumber);
  }else{
    fAnaLoop->AnaLoop(fFileName.Data(), fEventNumber);
  }    

  return 0;
}

void TArtAnaLoopStatic::SetAnaLoop(TArtAnaLoop* analoop)
{
  //  if(fAnaLoop) delete fAnaLoop;
  fAnaLoop = analoop;
}
void TArtAnaLoopStatic::SetFileName(const char* filename)
{
  if(filename == 0){
    fFileName = "";
  }else{
    fFileName = filename;
  }
}
void TArtAnaLoopStatic::SetEventNumber(int eventnumber){fEventNumber = eventnumber;}
void TArtAnaLoopStatic::SetDirectory(TDirectory* directory)
{
  if(directory == 0){
    fDirectory = gDirectory;
  }else{
    fDirectory = directory;
  }
}  
TArtAnaLoop* TArtAnaLoopStatic::GetAnaLoop(){return fAnaLoop;}
const char* TArtAnaLoopStatic::GetFileName(){return fFileName.Data();}
int TArtAnaLoopStatic::GetEventNumber(){return fEventNumber;}
TDirectory* TArtAnaLoopStatic::GetDirectory(){return fDirectory;}

void TArtAnaLoopStatic::Clear()
{
  fAnaLoop = 0;
  fFileName = "";
  fEventNumber = -1;
  fDirectory = 0;
}

TArtAnaLoopStatic::~TArtAnaLoopStatic()
{
  Clear();
}

